﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using IdeaBlade.EntityModel;

namespace WindowsStoreDevTour {
  /// <summary>
  /// A data service to abstract the EntityManager. Consider using an IoC container to inject insted of a static singleton
  /// </summary>
  public class DataService {
    private static DataService _instance;
    private readonly NorthwindIBEntities _entityManager;

    protected DataService() {
      _entityManager = new NorthwindIBEntities();
    }

    public static DataService Instance {
      get { return _instance ?? (_instance = new DataService()); }
    }

    public Task<IEnumerable<Customer>> GetAllCustomersAsync() {
      return _entityManager.Customers.OrderBy(x => x.CompanyName).ExecuteAsync();
    }

    public Task<IEnumerable<Customer>> FindCustomersAsync(string queryText) {
      return _entityManager.Customers
          .Where(x => x.CompanyName.Contains(queryText))
          .OrderBy(x => x.CompanyName)
          .ExecuteAsync();
    }

    public async Task<Customer> GetCustomerAsync(Guid id) {
      var customers = await _entityManager.Customers
                                .Where(x => x.CustomerID == id)
                                .ExecuteAsync();

      return customers.FirstOrDefault();
    }

    public Task SaveAsync() {
      return _entityManager.SaveChangesAsync();
    }

    public void Discard() {
      _entityManager.RejectChanges();
    }

    public bool HasChanges() {
      return _entityManager.HasChanges();
    }

    public event EventHandler<EntityChangedEventArgs> EntityChanged {
      add { _entityManager.EntityChanged += value; }
      remove { _entityManager.EntityChanged -= value; }
    }
  }
}