﻿using System.Collections.ObjectModel;
using System.ComponentModel;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace WindowsStoreDevTour {
  public sealed partial class ListPage : INotifyPropertyChanged {
    private ObservableCollection<Customer> _customers;
    private bool _isBusy;
    private string _searchText;
    private Customer _selectedCustomer;

    public ListPage() {
      InitializeComponent();

      // Consider using an MVVM framework such as Cocktail instead of Code-Behind.
      DataContext = this;
    }

    public bool IsBusy {
      get { return _isBusy; }
      private set {
        _isBusy = value;
        NotifyPropertyChange("IsBusy");
      }
    }

    public ObservableCollection<Customer> Customers {
      get { return _customers; }
      set {
        if (Equals(value, _customers)) return;
        _customers = value;
        NotifyPropertyChange("Customers");
      }
    }

    public Customer SelectedCustomer {
      get { return _selectedCustomer; }
      set {
        if (Equals(value, _selectedCustomer)) return;
        _selectedCustomer = value;
        NotifyPropertyChange("SelectedCustomer");

        NavigateToDetailPage();
      }
    }

    public string SearchText {
      get { return _searchText; }
      set {
        if (value == _searchText) return;
        _searchText = value;
        NotifyPropertyChange("SearchText");
      }
    }

    #region INotifyPropertyChanged Members

    public event PropertyChangedEventHandler PropertyChanged;

    #endregion

    protected override void OnNavigatedTo(NavigationEventArgs e) {
      base.OnNavigatedTo(e);

      Start();
    }

    public async void Start() {
      try {
        IsBusy = true;

        var customers = await DataService.Instance.GetAllCustomersAsync();
        Customers = new ObservableCollection<Customer>(customers);
      } finally {
        IsBusy = false;
      }
    }

    private void NavigateToDetailPage() {
      Frame.Navigate(typeof(DetailPage), SelectedCustomer.CustomerID);
    }

    private void NotifyPropertyChange(string propertyName) {
      var handler = PropertyChanged;
      if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
    }

    private void GoBack(object sender, RoutedEventArgs e) {
      Frame.GoBack();
    }

    private async void Search(object sender, RoutedEventArgs e) {
      try {
        IsBusy = true;

        if (string.IsNullOrEmpty(SearchText)) {
          Start();
          return;
        }

        var customers = await DataService.Instance.FindCustomersAsync(SearchText);
        Customers = new ObservableCollection<Customer>(customers);
      } finally {
        IsBusy = false;
      }
    }
  }
}
